@@ -4,7 +4,7 @@ methods that act upon a transfer function to gain functions such as the
 mass function.
 '''
 
-version = '1.1.5'
+version = '1.1.6'
 
 ###############################################################################
 # Some Imports
@@ -20,6 +20,7 @@ import cosmography
 import tools
 from fitting_functions import fits
 
+#TODO: possibly implement weighting by survey volume: n(M,z1,z2) = \int n(M,z)*V(z) dz / \int V(z) dz
 ###############################################################################
 # The Class
 ###############################################################################
@@ -43,19 +44,19 @@ class Perturbations(object):
         growth:        The growth factor for the given cosmology and redshift
         power:         The natural log of the normalised power spectrum for the given cosmology (at lnk)
         n_eff:         Effective spectral index at the radius of a halo
-        M:             The masses at which analysis is performed. (not log) [M/h]
+        M:             The masses at which analysis is performed. (not log) [M_sun/h]
         fsigma:        The multiplicity function, or fitting function at M
         dndm:          The comoving number density of halos in mass interval M [h**3/Mpc**3]
         dndlnm:        The comoving number density of halos in log mass interval M [h**3/Mpc**3]
         dndlog10m:     The comoving number density of halo in log10 mass interval M [h**3/Mpc**3]
         ngtm:          Comoving number density of halos > M [h**3/Mpc**3]
         nltm:          Comoving number density of halos < M [h**3/Mpc**3]
-        mgtm:          Comoving mass density of halos > M [h**3/Mpc**3]
-        mltm:          Comoving mass density of halos < M [h**3/Mpc**3]
+        mgtm:          Comoving mass density of halos > M [M_sun h**3/Mpc**3]
+        mltm:          Comoving mass density of halos < M [M_sun h**3/Mpc**3]
         how_big:       The requisite size of a simulation box, L, to have at least one halo > M [Mpc/h]
         
     Input:
-        M:             A vector of floats containing the log10(Solar Masses) at which to perform analysis.    
+        M:             A vector of floats containing the log10(Solar Masses/h) at which to perform analysis.    
                        Default: M = np.linspace(10,15,501)
                        
         transfer_file: Either a string pointing to a file with a CAMB-produced transfer function,
@@ -76,10 +77,10 @@ class Perturbations(object):
                        Default: mf_fit = 'ST'
                        
                        Options:                               
-                            1. 'PS': Press-Schechter Approach
-                            2. 'ST': Sheth-Tormen
-                            3. 'Jenkins': Jenkins empirical fit
-                            4. 'Warren': Warren empirical fit
+                            1. 'PS': Press-Schechter form from 1974
+                            2. 'ST': Sheth-Mo-Tormen empirical fit from 2001
+                            3. 'Jenkins': Jenkins empirical fit from 2001
+                            4. 'Warren': Warren empirical fit from 2006
                             5. 'Reed03': Reed empirical from 2003
                             6. 'Reed07': Reed empirical from 2007
                             7. 'Tinker': Tinker empirical from 2008
@@ -90,11 +91,14 @@ class Perturbations(object):
                             12. 'Angulo': Angulo 2012
                             13. 'Angulo_Bound': Angulo sub-halo function 2012
                             14. "Bhattacharya": Bhattacharya empirical fit 2011
-                            15. "Behroozi": Behroozi extension to Tinker for high-z
+                            15. "Behroozi": Behroozi extension to Tinker for high-z 2013
                             16. 'user_model': A user-input string function
         
-        delta_vir:     The virial overdensity for the halo definition
-                       Default: delta_vir = 200.0
+        delta_wrt:     Defines what the overdensity of a halo is with respect to, can take 'mean' or 'crit'
+                       Default: 'mean' 
+                       
+        delta_halo:    The overdensity for the halo definition, with respect to delta_wrt
+                       Default: delta_halo = 200.0
                        
         user_fit:      A string defining a mathematical function in terms of 'x', used as the fitting function,
                        where x is taken as sigma. Will only be applicable if mf_fit == "user_model".
@@ -107,8 +111,6 @@ class Perturbations(object):
                        If False, will use function to calculate all values specified in M (may give ridiculous results)
                        Default: True
                        
-        R:             The distances at which the dark matter correlation function is calculated in Mpc/h
-                       Default: np.linspace(1, 200, 200)
                        
         **kwargs:      There is a placeholder for any additional cosmological parameters, or camb
                        parameters, that one wishes to include. Parameters that aren't used won't
@@ -166,7 +168,7 @@ class Perturbations(object):
                  transfer_file=None,
                  z=0.0,
                  wdm_mass=None, k_bounds=[0.0000001, 20000.0],
-                 delta_vir=200.0, user_fit='', transfer_fit='CAMB',
+                 delta_halo=200.0, delta_wrt='mean', user_fit='', transfer_fit='CAMB',
                  cut_fit=True, ** kwargs):
         """
         Initializes the cosmology for which to perform the perturbation analysis.      
@@ -220,7 +222,8 @@ class Perturbations(object):
                         "Watson_FoF", "Watson", "Crocce", "Courtin", "Bhattacharya", "Behroozi", "user_model"]
 
         self.update(M=M, mf_fit=mf_fit, k_bounds=k_bounds, transfer_file=transfer_file, wdm_mass=wdm_mass,
-                     delta_vir=delta_vir, user_fit=user_fit, z=z, transfer_fit=transfer_fit, cut_fit=cut_fit, ** kwargs)
+                     delta_halo=delta_halo, delta_wrt=delta_wrt, user_fit=user_fit, z=z,
+                     transfer_fit=transfer_fit, cut_fit=cut_fit, ** kwargs)
 
     def update(self, **kwargs):
         """
@@ -278,8 +281,10 @@ class Perturbations(object):
                 self.z = kwargs['z']
             elif key is "wdm_mass":
                 self.wdm_mass = kwargs['wdm_mass']
-            elif key is 'delta_vir':
-                self.delta_vir = kwargs['delta_vir']
+            elif key is 'delta_halo':
+                self._delta_halo_base = kwargs['delta_halo']
+            elif key is 'delta_wrt':
+                self.delta_wrt = kwargs['delta_wrt']
             elif key is "user_fit":
                 self.user_fit = kwargs['user_fit']
             elif key is "transfer_fit":
@@ -366,6 +371,9 @@ class Perturbations(object):
 
     @property
     def M(self):
+        """
+        Mass in units of M_sun/h
+        """
         return self.__M
 
     @M.setter
@@ -464,26 +472,45 @@ class Perturbations(object):
         self.__wdm_mass = val
 
     @property
-    def delta_vir(self):
+    def _delta_halo_base(self):
         """
-        The virial overdensity with respect to the background
+        The overdensity given by user: undefined until it is given with respect to something
         """
-        return self.__delta_vir
+        return self.__delta_halo_base
 
-    @delta_vir.setter
-    def delta_vir(self, val):
+    @_delta_halo_base.setter
+    def _delta_halo_base(self, val):
         try:
             val = float(val)
         except ValueError:
-            raise ValueError("delta_vir must be a number: ", val)
+            raise ValueError("delta_halo must be a number: ", val)
 
         if val <= 0:
-            raise ValueError("delta_vir must be > 0 (", val, ")")
+            raise ValueError("delta_halo must be > 0 (", val, ")")
+        if val > 10000:
+            raise ValueError("delta_halo must be < 10,000 (", val, ")")
+
+        #Set the value
+
+        self.__delta_halo_base = val
 
         #Delete stuff dependent on it
-        del self.fsigma
-        self.__delta_vir = val
+        del self.delta_halo
 
+    @property
+    def delta_wrt(self):
+        """
+        Defines what the delta_halo is with respect to
+        """
+        return self.__delta_wrt
+
+    @delta_wrt.setter
+    def delta_wrt(self, val):
+        if val not in ['mean', 'crit']:
+            raise ValueError("delta_wrt must be either 'mean' or 'crit' (", val, ")")
+
+        self.__delta_wrt = val
+        del self.delta_halo
 
     @property
     def z(self):
@@ -575,6 +602,26 @@ class Perturbations(object):
 
     #--------------------------------  START NON-SET PROPERTIES ----------------------------------------------
     @property
+    def delta_halo(self):
+        """ Overdensity of a halo wrt mean density"""
+        try:
+            return self.__delta_halo
+        except:
+            if self.delta_wrt == 'mean':
+                self.__delta_halo = self._delta_halo_base
+
+            elif self.delta_wrt == 'crit':
+                self.__delta_halo = self._delta_halo_base / cosmography.omegam_z(self.z, self.cosmo_params['omegam'], self.cosmo_params['omegav'],
+                                                                                self.cosmo_params['omegak'])
+            return self.__delta_halo
+    @delta_halo.deleter
+    def delta_halo(self):
+        try:
+            del self.__delta_halo
+            del self.fsigma
+        except:
+            pass
+    @property
     def dlogM(self):
         return self.M[1] - self.M[0]
 
@@ -875,7 +922,7 @@ class Perturbations(object):
         try:
             return self.__fsigma
         except:
-            fits_class = fits(self.M, self.n_eff, self.mf_fit, self.sigma, self.cosmo_params['delta_c'], self.z, self.delta_vir, self.user_fit, self.cut_fit)
+            fits_class = fits(self.M, self.n_eff, self.mf_fit, self.sigma, self.cosmo_params['delta_c'], self.z, self.delta_halo, self.cosmo_params, self.user_fit, self.cut_fit)
             nufnu = fits_class.nufnu()
             self.__fsigma = nufnu()
             return self.__fsigma
@@ -971,7 +1018,7 @@ class Perturbations(object):
             dlnsdlnm = tools.dlnsdlnm(np.exp(m_upper), sigma_0, self._power_0, self.lnk, self.cosmo_params['mean_dens'])
             n_eff = tools.n_eff(dlnsdlnm)
             fsigma = fits(m_upper, n_eff, self.mf_fit, sigma, self.cosmo_params['delta_c'],
-                          self.z, self.delta_vir, self.user_fit, cut_fit=True).nufnu()()
+                          self.z, self.delta_halo, self.cosmo_params, self.user_fit, cut_fit=True).nufnu()()
             #fsigma = nufnu()
             dndm = fsigma * self.cosmo_params['mean_dens'] * np.abs(dlnsdlnm) / np.exp(m_upper) ** 2
             mf = np.log(np.exp(m_upper) * dndm)
@@ -1002,7 +1049,7 @@ class Perturbations(object):
             dlnsdlnm = tools.dlnsdlnm(np.exp(m_lower), sigma_0, self._power_0, self.lnk, self.cosmo_params['mean_dens'])
             n_eff = tools.n_eff(dlnsdlnm)
             fsigma = fits(m_lower, n_eff, self.mf_fit, sigma, self.cosmo_params['delta_c'],
-                          self.z, self.delta_vir, self.user_fit, cut_fit=True).nufnu()()
+                          self.z, self.delta_halo, self.cosmo_params, self.user_fit, cut_fit=True).nufnu()()
             #fsigma = nufnu()
             dndm = fsigma * self.cosmo_params['mean_dens'] * np.abs(dlnsdlnm) / np.exp(m_lower) ** 2
             mf = np.log(np.exp(m_lower) * dndm)
